/******************************************************************************
  (c) Copyright 2002-2008, 1060 Research Ltd                                   

  This Software is licensed to You, the licensee, for use under the terms of   
  the 1060 Public License v1.0. Please read and agree to the 1060 Public       
  License v1.0 [www.1060research.com/license] before using or redistributing   
  this software.                                                               

  In summary the 1060 Public license has the following conditions.             
  A. You may use the Software free of charge provided you agree to the terms   
  laid out in the 1060 Public License v1.0                                     
  B. You are only permitted to use the Software with components or applications
  that provide you with OSI Certified Open Source Code [www.opensource.org], or
  for which licensing has been approved by 1060 Research Limited.              
  You may write your own software for execution by this Software provided any  
  distribution of your software with this Software complies with terms set out 
  in section 2 of the 1060 Public License v1.0                                 
  C. You may redistribute the Software provided you comply with the terms of   
  the 1060 Public License v1.0 and that no warranty is implied or given.       
  D. If you find you are unable to comply with this license you may seek to    
  obtain an alternative license from 1060 Research Limited by contacting       
  license@1060research.com or by visiting www.1060research.com                 

  NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC  
  LICENSE V1.0 FOR DETAILS                                                     

  THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ     
  THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license      
 *****************************************************************************/
package org.netkernel.container;

import java.util.Iterator;
import org.netkernel.container.config.*;
import org.netkernel.request.*;
import org.netkernel.urii.*;

/**
 * Interface to the NetKernel microkernel instance.
 * @author  tab
 */
public interface IKernel extends IAsyncRequestee, ISynchronousRequestee
{
	/** Register a new space to be managed by the kernel. Lifecycle of the space
	 * will be managed and it is made available through the getSpace() method
	 * @param aSpace the space to be registered
	 * @throws INetKernelException thrown if the space cannot be registered
	 * @throws IllegalStateException thrown if the kernel is not paused
	 */
	void register(ISpaceWithIdentity aSpace) throws INetKernelException, IllegalStateException;
	
	/** Deregister a space that has previously been registered
	 * @param aSpace the space to be deregistered
	 * @throws INetKernelException thrown if the space fails to deregister successfully, it
	 *  will still be removed the kernels working set though.
	 * @throws IllegalStateException thrown if the kernel is not paused
	 */
	void deregister(ISpaceWithIdentity aSpace)  throws INetKernelException, IllegalStateException;
	
	/** Add a listener to be informed when any spaces are commissioned or decommissioned
	 * @param aListener the listener
	 */
	void addSpaceListener(ISpaceListener aListener);
	
	/** Removes a listener to be informed when any spaces are commissioned or decommissioned
	 * @param aListener
	 */
	void removeSpaceListener(ISpaceListener aListener);
	
	/** Returns an iterator over the working set of registered spaces
	 * @return the iterator
	 */
	Iterator<ISpaceWithIdentity> getSpaces();
	
	/** Retrieve a space by its identity.
	 * @param aIdentifier the identity of the space
	 * @param aMin optional minimum version specifier
	 * @param aMa		// TODO Auto-generated method stub
		
x optional maximum version specifier
	 * @return the most recent by version space which matches the given constrains or null
	 */
	ISpaceWithIdentity getSpace(IIdentifier aIdentifier, IVersion aMin, IVersion aMax);
	
	/** Returns the external configuration instance being used configure the kernel
	 * @return the instance
	 */
	IConfiguration getConfiguration();
	
	/**  Add a listener to be informed when the configuration changes 
	 * @param aListener the listener
	 */
	void addConfigurationListener(IConfigurationListener aListener);
	
	/** Removes a listener to be informed when the configuration changes 
	 * @param aListener the listener
	 */
	void removeConfigurationListener(IConfigurationListener aListener);	
	
	/** Returns the optional external listener which is notified when various kernel events happen
	 * @return the instance or null if none is specified
	 */
	IKernelListener getKernelListener();
	
	/** Returns the external logger instance being used to receive kernel logging events
	 * @return the instance
	 */
	ILogger getLogger();
	
	public static final int S_PAUSED=0;
	public static final int S_READY=1;
	public static final int S_OPEN=2;
	public static final int S_CLOSED=3;
	public static final int S_FINAL=4;
	
	void setState(int aState) throws IllegalArgumentException;

	/** Returns the current state of the kernel
	 * @return an integer as defined by the S_* constants
	 */
	int getState();

	/**	Returns true if the debug flag has been set on the kernel and the tradeoff of capturing
	 *  useful debug information over performance has been enabled
	 * @return true/false
	 */
	boolean inDebug();
	
	/** Returns the representation cache instance
	 * @return
	 */
	IRepresentationCache getRepresentationCache();
	
	/** Returns the resolution cache instance
	 * @return
	 */
	IResolutionCache getResolutionCache();
	
	/* Returns the request timer instance
	 */
	ITimer getTimer();
	
	/** Rather than IRequestee this interface method is used for inject of external requests
	 * @param aRequest the request to issue
	 * @param aTransportInstance the instance of the transport injecting the request
	 * @param aTransportSpace the space in which the transport resides
	 * @return the response to the request
	 * @throws INetKernelException thrown if the request handling fails.
	 */
	IResponse synchronousExternalRequest(IRequest aRequest, Object aTransportInstance, ISpace aTransportSpace) throws INetKernelException;
	
	/** Rather than IRequestee this interface method is used for inject of external requests. Response is
	 * returned back to aRequest.getRequestor().
	 * @param aRequest the request to issue
	 * @param aTransportInstance the instance of the transport injecting the request
	 * @param aTransportSpace the space in which the transport resides
	 */
	void onAsyncExternalRequest(IRequest aRequest, Object aTransportInstance, ISpace aTransportSpace);

	/** When in the closed state this request will cause any blocked requests from the given transport instance to be interrupted and return
	 * immediately with an exception
	 * @param aTransportInstance
	 */
	void interruptExternalRequests(Object aTransportInstance);
	
	//TODO request control interface to kernel
	//IKernelRequest[] getActiveRequests();
	//boolean killActiveRequest(IKernelRequest aRequest);
}