/******************************************************************************
  (c) Copyright 2002-2008, 1060 Research Ltd                                   

  This Software is licensed to You, the licensee, for use under the terms of   
  the 1060 Public License v1.0. Please read and agree to the 1060 Public       
  License v1.0 [www.1060research.com/license] before using or redistributing   
  this software.                                                               

  In summary the 1060 Public license has the following conditions.             
  A. You may use the Software free of charge provided you agree to the terms   
  laid out in the 1060 Public License v1.0                                     
  B. You are only permitted to use the Software with components or applications
  that provide you with OSI Certified Open Source Code [www.opensource.org], or
  for which licensing has been approved by 1060 Research Limited.              
  You may write your own software for execution by this Software provided any  
  distribution of your software with this Software complies with terms set out 
  in section 2 of the 1060 Public License v1.0                                 
  C. You may redistribute the Software provided you comply with the terms of   
  the 1060 Public License v1.0 and that no warranty is implied or given.       
  D. If you find you are unable to comply with this license you may seek to    
  obtain an alternative license from 1060 Research Limited by contacting       
  license@1060research.com or by visiting www.1060research.com                 

  NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC  
  LICENSE V1.0 FOR DETAILS                                                     

  THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ     
  THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license      
 *****************************************************************************/
package org.netkernel.request;

import java.util.List;

/** A flexible multimap of objects indexed by either a java.lang.String key value
 * or a int positional parameter.
 *
 * Used by IRequest and IResponseMeta to pass request header and response metadata information.
 *
 * @author Tony Butterfield
 * @see org.netkernel.request.IRequest
 * @see org.netkernel.request.IResponseMeta
 */
public interface IRequestResponseFields
{
	/** Returns the number of fields.
   *
   * @return Size of the map.
	 */
	int size();
	
	/** Returns the key value at the given index position.
   *
   * @return Key value at an index position.
	 */
	String getKey(int aIndex);
		
	/** Returns the object at the given index position.
   *
   * @return Object at an index position.
	 */
	Object getValue(int aIndex);
	
	/** Return true if the given field is sticky
	 */
	boolean isSticky(int aIndex);
	
	/** Returns the object for a given key value.
   *
   * @return Object for the given key value.
	 */
	Object getValue(String aKey);
	
   /** @return boolean true if given key is sticky
	 */
	boolean isSticky(String aKey);
	
	/** Returns a list of all values for a given key
	 */
	List<Object> getValues(String aKey);
	
}
