package org.netkernel.request.impl;

import java.util.*;
import org.netkernel.request.IRequestResponseFields;

public class RequestResponseFieldsImpl implements IRequestResponseFields
{
	private Object[] mData;
	private int mIndex;
	
	public static final RequestResponseFieldsImpl EMPTY = new RequestResponseFieldsImpl(0);
	private static final int LEN=2;
	
	public RequestResponseFieldsImpl(int aInitialSize)
	{	mData= new Object[aInitialSize*LEN];
	}
	
	public RequestResponseFieldsImpl(IRequestResponseFields aOther)
	{	if (aOther instanceof RequestResponseFieldsImpl)
		{	RequestResponseFieldsImpl other = (RequestResponseFieldsImpl)aOther;
			mIndex = other.mIndex;
			mData = new Object[other.mData.length];
			System.arraycopy(other.mData, 0, mData, 0, mIndex);
		}
		else
		{	int size=aOther.size();
			mIndex=size*LEN;
			mData = new Object[mIndex+LEN*2];
			int j=0;
			for (int i=0; i<size; i++)
			{	mData[j++]=aOther.getKey(i);
				mData[j++]=aOther.getValue(i);
				mData[j++]=aOther.isSticky(i);
			}
		}
	}
	
	/** Put in a new value, will replace all existing values with same key
	 * @param aIdentifier identifier to match the source request
	 * @param aRepresentation representation in response
	 */
	public void put(String aKey, Object aRepresentation)
	{	int putIndex=mIndex;
		for (int i=0; i<mIndex; i+=LEN)
		{	if (mData[i].equals(aKey))
			{	if (putIndex==mIndex)
				{	putIndex=i;
				}
				else
				{	//we have more than one
					remove(aKey);
					putIndex=mIndex;
					break;
				}
			}
		}
		if (putIndex==mIndex)
		{	//resize if needed
			if (mIndex==mData.length)
			{	int newSize=(mIndex==0)?LEN*2:mIndex*2;
				Object[] newData = new Object[newSize];
				System.arraycopy(mData, 0, newData, 0, mIndex);
				mData=newData;
			}
			mIndex+=LEN;
		}
		
		mData[putIndex++]=aKey;
		mData[putIndex]=aRepresentation;
	}
	
	/** Add a new value, this will be added in addition to any existing values
	 * for the given key
	 * @param aKey
	 * @param aRepresentation
	 */
	public void add(String aKey, Object aRepresentation)
	{
		//resize if needed
		if (mIndex==mData.length)
		{	int newSize=(mIndex==0)?LEN*2:mIndex*2;
			Object[] newData = new Object[newSize];
			System.arraycopy(mData, 0, newData, 0, mIndex);
			mData=newData;
		}
		mData[mIndex++]=aKey;
		mData[mIndex++]=aRepresentation;
	}
	
	/** Remove first instance of the given key
	 * @param aKey
	 * @return the removed value
	 */
	public Object remove(String aKey)
	{	Object removed=null;
		for (int i=0; i<mIndex; i+=LEN)
		{	if (mData[i].equals(aKey))
			{	removed=mData[i+1];
				System.arraycopy(mData, i+LEN, mData, i,mIndex-i-LEN);
				mIndex-=LEN;
				break;
			}
		}
		return removed;
	}	

	public String getKey(int aIndex)
	{	return (String)mData[aIndex*LEN];
	}

	public Object getValue(int aIndex)
	{	return mData[aIndex*LEN+1];
	}

	public Object getValue(String aKey)
	{	Object result=null;
		for (int i=0; i<mIndex; i+=LEN)
		{	if (mData[i].equals(aKey))
			{	result=mData[i+1];
				break;
			}
		}	
		return result;
	}
	
	public List getValues(String aKey)
	{	List result = new ArrayList();
		for (int i=0; i<mIndex; i+=LEN)
		{	if (mData[i].equals(aKey))
			{	result.add(mData[i+1]);
			}
		}
		return result;
	}
	
	public int size()
	{	return mIndex/LEN;
	}
	
	public boolean equals(Object aOther)
	{	boolean equal = aOther instanceof RequestResponseFieldsImpl;
		if (equal)
		{	RequestResponseFieldsImpl other = (RequestResponseFieldsImpl)aOther;
			equal=Arrays.equals(mData, other.mData);
		}
		return equal;
	}

	public boolean isSticky(int aIndex)
	{	return false;
	}

	public boolean isSticky(String aKey)
	{	return false;
	}

}
